# handle the tal: namespace

tal_ns = 'http://xml.zope.org/namespaces/tal'

from pax.paxtransform import AttributeHandler, StopTransform
from pax import XML
from pax.backwards_compatibility import *
import re

def text_or_structure(value):
    return split_spec(value, 'text', 'structure')

def split_spec(value, *names):
    for name in names:
        if value.startswith(name + ' '):
            return name, value[len(name)+1:]
    return names[0], value

def remove_attrs_till(name, element):
    try:
        attrs = element.attributes[tal_ns]
    except KeyError:
        return # no tal attributes
    for n, h in tal_handler.handlers:
        try:
            del attrs[n]
        except KeyError:
            pass
        if n == name:
            break

class Structure(object):
    def __init__(self, text):
        self.text = text

    def __xml__(self, *a):
        return self.text

multiple_re = re.compile('(?<!;);(?!;)')

def split_multiple(s):
    """split multiple semicolon-separated values
    two semicolons should be considered an escape"""
    parts = [sub.strip().replace(';;', ';') for sub in multiple_re.split(s)]
    if len(parts) > 1 and not parts[-1].strip():
        del parts[-1] # It ended in a semicolon
    return parts


def handle_define(element, context, value):
    for item in split_multiple(value):
        scope, expr = split_spec(item, 'local', 'global')
        name, expr = expr.split(' ', 1)
        if scope == 'global':
            context.engine.setGlobal(name, context.engine.evaluateValue(expr))
        else:
            context.engine.setLocal(name, context.engine.evaluateValue(expr))
    return element

def handle_condition(element, context, value):
    if context.engine.evaluateBoolean(value):
        return element
    raise StopTransform

def handle_repeat(element, context, value):
    remove_attrs_till('repeat', element)
    name, value = value.split(' ', 1)
    result = ()
    iterator = context.engine.setRepeat(name, value)
    while iterator.next():
        result += (context.tr_engine.transform(element, context),)
    raise StopTransform, (result,)

def get_text_or_structure(value, context):
    mode, value = text_or_structure(value)
    content = context.engine.evaluateText(value)
    if content is not context.engine.getDefault() and content and mode == 'structure':
        content = Structure(content)
    return content

def handle_replace(element, context, value):
    content = get_text_or_structure(value, context)
    if content is context.engine.getDefault():
        return element
    raise StopTransform, content

def handle_content(element, context, value):
    content = get_text_or_structure(value, context)
    if content is context.engine.getDefault():
        return element
    element.children = [content]
    return element

def handle_attributes(element, context, value):
    for item in split_multiple(value):
        spec, expr = item.split(' ', 1)
        if ':' in spec:
            ns, name = spec.split(':', 1)
        else:
            ns, name = element.ns, spec
        value = context.engine.evaluateText(expr)
        if value is not context.engine.getDefault():
            nsattrs = element.attributes.setdefault(ns, {})
            if value:
                nsattrs[name] = value
            else:
                try:
                    del nsattrs[name]
                except KeyError:
                    pass
    return element

def handle_omit_tag(element, context, value):
    if value and not context.engine.evaluateBoolean(value):
        return element
    return element.children

def tal_start(element, context):
    context.engine.beginScope()
    context.engine.setLocal('attrs', element.attributes.get(element.ns, {}))
    return element

def tal_finish(element, context):
    try:
        del element.attributes[tal_ns]
        for dprefix, duri in element.nsdecls.items():
            if duri == tal_ns:
                del element.nsdecls[dprefix]
    except (AttributeError, KeyError):
        pass
    try:
        context.engine.endScope()
    except IndexError:
        pass
    if getattr(element, 'ns', None) == tal_ns:
        return element.children
    else:
        return element


tal_handler = AttributeHandler('tal', tal_ns)
#tal_handler.add_handler('on_error', handle_on_error)
tal_handler.add_handler('define', handle_define)
tal_handler.add_handler('condition', handle_condition)
tal_handler.add_handler('repeat', handle_repeat)
tal_handler.add_handler('content', handle_content)
tal_handler.add_handler('replace', handle_replace)
tal_handler.add_handler('attributes', handle_attributes)
tal_handler.add_handler('omit-tag', handle_omit_tag)
tal_handler.preprocess = tal_start
tal_handler.postprocess = tal_finish

__all__ = ('tal_handler',)

if __name__ == '__main__':
    import sys, pax, pax.paxtransform
    tal_tr_engine = pax.paxtransform.Engine()
    tal_tr_engine.add_handler(tal_ns, tal_handler)
    import DummyEngine
    class DumbContext:
        def __init__(self, tr_engine):
            self.engine = DummyEngine.DummyEngine()
            self.tr_engine = tr_engine
    if len(sys.argv) > 1:
        srcfile = open(sys.argv[1])
    else:
        srcfile = sys.stdin
        sys.stderr.write('Reading source from stdin\n')
    source = pax.xml2pax(srcfile.read())
    print pax.XML(tal_tr_engine.transform(source, DumbContext(tal_tr_engine)))
